import {
  Metadata, IDictionary, IEventParam, Http, Router, Loader,
} from '@zeedhi/core';
import ZdColorPicker from '@/components/colorPicker/ZdColorPicker';
import {
  Button, Form, Card, Tabs, Text, IComponentRender, AlertService,
  Switch, Container, FileInput, Textarea, TextInput, CollapseCard,
  Select, SelectMultiple, Chip, DialogService, Col, Modal, ModalService, Icon,
} from '@zeedhi/common';
import { IterableComponentRender } from '@zeedhi/teknisa-components-common';

import { getRouteParams } from '../router';

export class ChecklistController {
  public fromTypescriptModal!: Modal;

  public modalController = Loader.getInstance('ModalControllerAddAtividade');

  public modalController1 = Loader.getInstance('ModalControllerAddProcedimento');

  // Controla a visibilidade dos botões de "Adicionar", "Editar", "Copiar" e "excluir" checklist devido ao nível do acesso do usuário
  public isVisibleElement: boolean = !(localStorage.getItem('idpermissschecklist') === 'V');

  public isDisabledClick: boolean = !(this.isVisibleElement);

  // public novoChecklistController = Loader.getInstance('NovoChecklistController');

  public dataFilter: IDictionary = {
    inputLocal: '',
    inputAtividades: '',
  };

  public dadosChip: IDictionary = {};

  public variavel: IDictionary = {};

  public titulo: IDictionary = {};

  public descricao: IDictionary = {};

  public conformidades: IDictionary = {};

  public modeloCriterio: IDictionary = {};

  public criterio: IDictionary = [];

  public renderizaCard() {
    const selectObjectCardOdin = Metadata.getInstance<Card>('segundaColunaTabs');
    selectObjectCardOdin.children = [];
    Http.post('/getDadosCard', this.dataFilter).then((response) => {
      const dadosTotal: IDictionary = [];
      response.data.dataset.dadosLocal.forEach((dados: IDictionary) => {
        // -----------------------------
        if (Object.keys(dadosTotal).includes(dados.NMTIPOLOCALCLI)) {
          dadosTotal[dados.NMTIPOLOCALCLI].push(this.geraCardInterno({
            nmChecklist: dados.NMCHECKLIST,
            nrChecklist: dados.NRCHECKLIST,
            descricao: dados.DSCHECKLIST,
            conformidades: dados.NRCRITICIDADECHK,
            pos: dados.NRCHECKLIST,
          }));
        } else {
          dadosTotal[dados.NMTIPOLOCALCLI] = [];
          dadosTotal[dados.NMTIPOLOCALCLI].push(this.geraCardInterno({
            nmChecklist: dados.NMCHECKLIST,
            descricao: dados.DSCHECKLIST,
            conformidades: dados.NRCRITICIDADECHK,
            pos: dados.NRCHECKLIST,
          }));
        }
        // ----------------------------
      });
      Object.keys(dadosTotal).forEach((key: string) => selectObjectCardOdin.children.push(this.geraCard(key, dadosTotal[key])));
    });
  }

  public renderizaCardAtividadesTeste() {
    Metadata.getInstance<IterableComponentRender>('iterable_parent').datasource.data = [];
    const dadosData: IDictionary = [];
    Http.post('/buscaDadosAtividadeseCriterios', this.dataFilter).then((response: IDictionary) => {
      response.data.dataset.dados.forEach((dados: IDictionary) => {
        const dadosAtividades = Object.keys(dados).filter((key: string) => key !== 'NMATIVIDADE' && key !== 'NRATIVIDADE' && key !== 'DSCORIDATIVIDADE');
        const tarefasatividade: IDictionary = [];
        dadosAtividades.forEach((key: string) => {
          tarefasatividade.push({
            NRTAREFA: dados[key].NRTAREFA,
            NMTAREFA: dados[key].NMTAREFA,
            CRITERIOS: dados[key].CRITERIOS,
            TAGS: dados[key].TAGS,
            DSPOPSERVICO: dados[key].DSPOPSERVICO,
          });
        });
        dadosData.push({
          NMATIVIDADE: dados.NMATIVIDADE,
          NRATIVIDADE: dados.NRATIVIDADE,
          DADOSDAATIVIDADE: tarefasatividade,
          DSCORIDATIVIDADE: dados.DSCORIDATIVIDADE,
        });
      });
      Metadata.getInstance<IterableComponentRender>('iterable_parent').datasource.data = dadosData as IDictionary[];
    });
  }

  public informacoesTarefas(pai: string) {
    const selectObjectIterable = Metadata.getInstance<IterableComponentRender>('iterable_parent').datasource.data;
    const dados = selectObjectIterable.filter((infos: IDictionary) => infos.NRATIVIDADE === Number(pai));
    dados.forEach((dadosFiltro: IDictionary) => {
      dadosFiltro.DADOSDAATIVIDADE.forEach((linha: IDictionary) => {
        const criterio = linha.CRITERIOS.map((infoCriterio: any) => infoCriterio.DSCRITERIOTAREFA);
        const tag = linha.TAGS.map((infoTag: any) => infoTag.DSTAGSTAREFA);
        this.criaModeloLinha(linha.NMTAREFA, criterio.join('; '), tag.join('; '), (linha.DSPOPSERVICO) ? linha.DSPOPSERVICO : '', this.contador);
        this.contador += 1;
        const selectObjectCard = Metadata.getInstance<CollapseCard>(`Collapscard_${pai}`);
        selectObjectCard!.children[0]!.children!.splice(selectObjectCard!.children[0]!.children!.length - 5, 0, this.modeloCriterio[0]);
        selectObjectCard!.children[0]!.children!.splice(selectObjectCard!.children[0]!.children!.length - 5, 0, this.modeloCriterio[1]);
        selectObjectCard!.children[0]!.children!.splice(selectObjectCard!.children[0]!.children!.length - 5, 0, this.modeloCriterio[2]);
        selectObjectCard!.children[0]!.children!.splice(selectObjectCard!.children[0]!.children!.length - 5, 0, this.modeloCriterio[3]);
        selectObjectCard!.children[0]!.children!.splice(selectObjectCard!.children[0]!.children!.length - 5, 0, this.modeloCriterio[4]);
      });
    });
  }

  // Função para inserir card externo na tela de (CheckList)
  public geraCard(titulo: string, dados: IComponentRender[] = []) {
    return {
      name: 'cardTotal',
      component: 'ZdCollapseCard',
      cssClass: 'spacing-between-cards',
      header: [
        {
          name: 'linhaCard',
          component: 'ZdRow',
          children: [
            {
              name: 'iconeCard',
              component: 'ZdText',
              text: '',
              cssClass: 'zd-pt-2 zd-pe-2',
              tag: 'v-icon',
            },
            {
              name: 'titulo',
              component: 'ZdText',
              text: `${titulo}`,
              cssClass: 'color-title-card ',
              style: {
                'font-weight': 'bold',
                'font-size': '16px',
              },
            },
          ],
        },
      ],
      children: dados,
    };
  }

  // função para inserir o card interno no geracard
  public geraCardInterno(variavel: IDictionary) {
    return {
      name: 'cardCentral',
      component: 'ZdCard',
      cssClass: 'spacing-between-cards',
      hover: true,
      elevation: 4,
      children: [
        {
          name: 'linhaCard',
          component: 'ZdRow',
          children: [
            {
              name: 'colunaCard',
              component: 'ZdCol',
              flat: true,
              cols: '4',
              children: [
                {
                  name: 'tituloCardInterno',
                  component: 'ZdText',
                  text: `${variavel.nmChecklist}`,
                  cssClass: 'sub-title-text-card',
                },
              ],
            },
            {
              name: 'primeiraColunaCardInterno',
              component: 'ZdCol',
              cols: '5',
              children: [
                {
                  name: 'descicaoCardInterno',
                  component: 'ZdText',
                  text: `${variavel.descricao}`,
                  cssClass: 'color-text-card',
                },
              ],
            },
            {
              name: 'terceiraColunaBotao',
              component: 'ZdCol',
              isVisible: true,
              cols: '3',
              children: [
                {
                  name: 'linhaBotao',
                  component: 'ZdRow',
                  cssClass: 'zd-justify-end',
                  isVisible: this.isVisibleElement,
                  children: [
                    {
                      name: `botao_${variavel.pos}`,
                      component: 'ZdButton',
                      icon: 'true',
                      iconName: 'mdi-content-copy',
                      events: {
                        click: '{{ChecklistController.copiaCards}}',
                      },
                    },
                    {
                      name: 'botao2',
                      component: 'ZdButton',
                      icon: 'true',
                      iconName: 'mdi-pencil',
                      to: `/editChecklist/${variavel.pos}`,
                    },
                    {
                      name: `botaoDelete_${variavel.pos}`,
                      component: 'ZdButton',
                      icon: 'true',
                      iconName: 'mdi-trash-can-outline',
                      events: {
                        click: '{{ChecklistController.deleteCard}}',
                      },
                    },
                  ],
                },
              ],
            },
          ],
        },
      ],
    };
  }

  // função para copiar cards internos(não apagar)
  public copiaCards({ component }: IEventParam<Button>) {
    // PARA COPIAR DO BANCO DE DADOS
    const checklist = component.name.split('_')[1];
    Http.post('/getDadosCopiaCard', { NRCHECKLIST: checklist, NRORG: 20 }).then(() => {
      this.renderizaCard();
    });
  }

  public getParent1(component: IEventParam<any>, nameComponent: string): any {
    if (component.component === nameComponent) {
      return component;
    }
    if (component.parent) {
      return this.getParent1(component.parent, nameComponent);
    }
    return null;
  }

  public getParent({ component }:{ component:any }, nameComponent: string): any {
    if (component.component === nameComponent) {
      return component;
    }
    if (component.parent) {
      return this.getParent(component.parent, nameComponent);
    }
    return null;
  }

  public criaFooter(pos: number): IDictionary {
    return [{
      name: `form_footer1${pos}`,
      component: 'ZdFooter',
      color: 'transparent',
      cssClass: 'zd-pt-3',
      grid: {
        cols: 2,
      },
      padless: true,
      children: [
        {
          name: 'botaoInformacaoForm',
          component: 'ZdButton',
          icon: true,
          cssClass: 'zd-pt-2 zd-pr-4',
          iconName: 'mdi-message-alert',
          events: {
            click: '{{ChecklistController.getDescricao}}',
          },
        },
        {
          name: 'botaoInformacaoForm2',
          component: 'ZdButton',
          label: 'Adicionar',
          outline: true,
          width: 80,
          events: {
            click: '{{ChecklistController.adicionaNovaTArefaCardAtividades}}',
          },
        },
      ],
    }];
  }

  // Cria uma nova linha padrão para cadastro de uma nova atividade
  public criaModeloLinha(tarefa: string, criterio: string, tag: string, descricao: string, pos: number) {
    this.modeloCriterio = [{
      name: `DadosTarefas${pos}`,
      component: 'ZdTextInput',
      label: 'Tarefa',
      cssClass: 'spacing-left-selects',
      readonly: true,
      value: `${tarefa}`,
      validations: {
        required: '{}',
      },
      grid: {
        cols: 4,
      },
    },
    {
      name: `DadosCriterio${pos}`,
      component: 'ZdTextInput',
      label: 'Critério de Não-Conformidades',
      cssClass: 'spacing-left-selects',
      readonly: true,
      value: `${criterio}`,
      validations: {
        required: '{}',
      },
      grid: {
        cols: 3,
      },
    },
    {
      name: `DadosTag${pos}`,
      readonly: true,
      component: 'ZdTextInput',
      label: 'Escopo de Serviço',
      value: `${tag}`,
      validations: {
        required: '{}',
      },
      grid: {
        cols: 3,
      },
    },
    {
      name: `form_footer1${pos}`,
      component: 'ZdFooter',
      cssClass: 'zd-pt-3',
      color: 'transparent',
      padless: true,
      children: [
        {
          name: 'botaoInformacaoForm1',
          component: 'ZdButton',
          icon: true,
          iconName: 'mdi-message-alert',
          events: {
            click: '{{ChecklistController.getDescricao}}',
          },
          grid: {
            cols: 1,
          },
        },
        {
          name: 'botaoExcluirForm1',
          component: 'ZdButton',
          icon: true,
          iconName: 'mdi-trash-can-outline',
          events: {
            click: '{{ChecklistController.deletaCriterio}}',
          },
          grid: {
            cols: 1,
          },
        },
      ],
    },
    {
      name: `textareaResultModal${pos}`,
      component: 'ZdTextarea',
      value: `${descricao}`,
      isVisible: false,
    }];
  }

  public criaModeloLinhaFixo(tarefa: string, criterio: string, tag: string, descricao: string, pos: number) {
    this.modeloCriterio = [{
      name: `DadosTarefas${pos}`,
      component: 'ZdTextInput',
      label: 'Tarefa',
      cssClass: 'spacing-left-selects',
      value: `${tarefa}`,
      validations: {
        required: '{}',
      },
      grid: {
        cols: 4,
      },
    },
    {
      name: `DadosCriterio${pos}`,
      component: 'ZdTextInput',
      label: 'Critério de Não-Conformidades',
      value: `${criterio}`,
      cssClass: 'spacing-left-selects',
      validations: {
        required: '{}',
      },
      grid: {
        cols: 3,
      },
    },
    {
      name: `DadosTag${pos}`,
      component: 'ZdTextInput',
      label: 'Escopo de Serviço',
      value: `${tag}`,
      validations: {
        required: '{}',
      },
      grid: {
        cols: 3,
      },
    },
    {
      name: `form_footer1${pos}`,
      component: 'ZdFooter',
      cssClass: 'zd-pt-3',
      color: 'transparent',
      padless: true,
      children: [
        {
          name: 'botaoInformacaoForm1',
          component: 'ZdButton',
          icon: true,
          iconName: 'mdi-message-alert',
          events: {
            click: '{{ChecklistController.getDescricao}}',
          },
          grid: {
            cols: 1,
          },
        },
        {
          name: 'botaoExcluirForm1',
          component: 'ZdButton',
          icon: true,
          iconName: 'mdi-trash-can-outline',
          events: {
            click: '{{ChecklistController.deletaCriterio}}',
          },
          grid: {
            cols: 1,
          },
        },
      ],
    },
    {
      name: `textareaResultModal${pos}`,
      component: 'ZdTextarea',
      value: `${descricao}`,
      isVisible: false,
    }];
  }

  public criaModeloLinhaModal(tarefa: string, criterio: string, tag: string, descricao: string, pos: number) {
    this.modeloCriterio = [{
      name: `DadosTarefas${pos}`,
      component: 'ZdTextInput',
      label: 'Tarefa',
      cssClass: 'spacing-left-selects',
      value: `${tarefa}`,
      validations: {
        required: '{}',
      },
      grid: {
        cols: 4,
      },
    },
    {
      name: `DadosCriterio${pos}`,
      component: 'ZdTextInput',
      label: 'Critério de Não-Conformidades',
      cssClass: 'spacing-left-selects',
      value: `${criterio}`,
      validations: {
        required: '{}',
      },
      grid: {
        cols: 3,
      },
    },
    {
      name: `DadosTag${pos}`,
      component: 'ZdTextInput',
      label: 'Escopo de Serviço',
      value: `${tag}`,
      validations: {
        required: '{}',
      },
      grid: {
        cols: 3,
      },
    },
    {
      name: `form_footer1${pos}`,
      component: 'ZdFooter',
      cssClass: 'zd-pt-3',
      color: 'transparent',
      padless: true,
      children: [
        {
          name: 'botaoInformacaoForm1',
          component: 'ZdButton',
          icon: true,
          iconName: 'mdi-message-alert',
          events: {
            click: '{{ChecklistController.getDescricao}}',
          },
          grid: {
            cols: 1,
          },
        },
        {
          name: 'botaoExcluirForm1',
          component: 'ZdButton',
          icon: true,
          iconName: 'mdi-trash-can-outline',
          events: {
            click: '{{ChecklistController.deletaCriterioModal}}',
          },
          grid: {
            cols: 1,
          },
        },
      ],
    },
    {
      name: `textareaResultModal${pos}`,
      component: 'ZdTextarea',
      value: `${descricao}`,
      isVisible: false,
    }];
  }

  // Função para troca titulo do botão ao acionar a tab Atividades e Critérios
  public trocaConteudoBotaoTabs() {
    const tabObject = Metadata.getInstance<Tabs>('tabsLocais');
    const buttonObject = Metadata.getInstance<Button>('botaoNovoChecklist');
    if (tabObject.activeTab === 0) {
      buttonObject.label = 'Nova Checklist';
    }
    if (tabObject.activeTab === 1) {
      buttonObject.label = 'Novo Item';
    }
  }

  // Fução para trocar conteudo e rota do botão ao trocar de aba
  public acaoBotaoAtividades({ component }: IEventParam<Button>) {
    if (component.label === 'Novo Item') {
      this.modalController.openHeaderFooterModal();
    } else {
      Router.replace('/cadastroNovoChecklist');
    }
  }

  // função para verificar onde esta pegando os componentes(Console.Log);
  public consoleLog(variavel: string) {
    const titulo = Metadata.getInstance<Text>('tituloNovaCheckList');
    titulo.text = variavel;
  }

  // função para tornar campo de descrição do Modal visivel ou não( Somente parte esta sendo utilizada);
  public isVisible() {
    const switchObject = Metadata.getInstance<Switch>('switch');
    const containerObject = Metadata.getInstance<Container>('container');
    const switchObject1 = Metadata.getInstance<Switch>('switch2');
    const containerObject1 = Metadata.getInstance<Container>('containerArquivo');
    if (switchObject.value) {
      containerObject.isVisible = true;
      switchObject1.disabled = true;
    } else if (switchObject1.value) {
      containerObject1.isVisible = true;
      switchObject.disabled = true;
    } else {
      containerObject.isVisible = false;
      containerObject1.isVisible = false;
      switchObject1.disabled = false;
      switchObject1.disabled = false;
    }
  }

  public showBasicAlert() {
    AlertService.show({ name: 'basic-alert', text: 'A basic alert' });
  }

  // adiciona nova Atividade(Modal);
  public salvarCardAtividades() {
    const selecLinhaNomeAtividade = Metadata.getInstance<Form>('rowLinhaNomeTarefa');
    const selectFormObject = Metadata.getInstance<Form>('formNovaAtividade');
    const validacao1 = selecLinhaNomeAtividade.validate();
    const validacao2 = selectFormObject.validate();
    const componentOfColor = Metadata.getInstance<Col>('BlocoDeCor');
    let colorASave: string;
    let msg: string;
    if (componentOfColor.children.length === 3) {
      colorASave = componentOfColor.children[2].color;
      msg = 'Selecione uma cor!';
    } else {
      colorASave = 'transparent';
      msg = 'Confirme uma cor antes de finalizar!';
    }
    if (validacao1 && validacao2 && (colorASave !== 'transparent')) {
      AlertService.show({
        name: 'basic-alert',
        text: 'Atividade Cadastrada!',
        top: true,
        left: false,
        timeout: 3000,
        color: 'success',
      });
      const indice = selectFormObject.children.map((nomesCampos: IDictionary) => nomesCampos.name);
      const teste = indice.filter((nome: string) => nome.indexOf('DadosTarefas') !== -1);
      const valor : IDictionary = [];
      teste.forEach((nome: string) => {
        const indiceNome = nome.substr(12);
        valor.push({
          NMTAREFA: selectFormObject.value[`DadosTarefas${indiceNome}`],
          CRITERIO: selectFormObject.value[`DadosCriterio${indiceNome}`].replace(';', ',').split(','),
          TAG: selectFormObject.value[`DadosTag${indiceNome}`].replace(';', ',').split(','),
          DESCRICAO: selectFormObject.value[`textareaResultModal${indiceNome}`],
        });
      });
      // todos dados sera enviado para o BackEnd atravez de um insert
      const todosOsDados = {
        NMATIVIDADE: Metadata.getInstance<TextInput>('atividade').value,
        TAREFAS: valor,
        COLOR: colorASave,
      };
      Http.post('/addNovaTarefaCardAtividades', { DADOS: todosOsDados, NRORG: 20 });
      this.modalController.closeHeaderFooterModal();
      // window.location.reload();
    } else if (!(validacao1 && validacao2)) {
      AlertService.show({
        name: 'basic-alert',
        text: 'Preencha todos os campos!',
        top: true,
        left: false,
        timeout: 3000,
        color: 'error',
      });
    } else {
      AlertService.show({
        name: 'basic-alert',
        text: msg,
        top: true,
        left: false,
        timeout: 3000,
        color: 'error',
      });
    }
  }

  public contador: number = 0;

  // Função para adicionar uma linha acima (MODAl)
  public addNovoTarefasCriterioTag({ component }:{ component:Button }) {
    const form: any = component!.parent!.getChildInstance('formNovaAtividade');
    this.criaModeloLinhaModal('', '', '', '', this.contador);
    this.contador += 1;
    form.children.push(this.modeloCriterio[0]);
    form.children.push(this.modeloCriterio[1]);
    form.children.push(this.modeloCriterio[2]);
    form.children.push(this.modeloCriterio[3]);
    form.children.push(this.modeloCriterio[4]);
  }

  // Remove uma linha selecionada tanto da tela de Atividades
  public deletaCriterio({ component }:{ component:Button }) {
    const linha = component!.parent!.parent!;
    const variavel = linha!.children.map((dados: IDictionary) => dados.name);
    const variavel1 = variavel.indexOf(`DadosTarefas${component!.parent!.name.substr(12)}`);
    // DADOS BANCO
    if (linha!.name !== 'formNovaAtividade') {
      const pai = Metadata.getInstance<IterableComponentRender>('iterable_parent');
      let dataPai = pai.datasource.data.filter((valor: IDictionary) => valor.NRATIVIDADE === Number(linha!.name.split('_')[1]));
      dataPai = dataPai.map((valor: IDictionary) => valor.DADOSDAATIVIDADE);
      const index = dataPai[0].findIndex((valor: IDictionary) => valor.NMTAREFA === linha!.children[variavel1].value);
      const nrCriteriosTarefas = dataPai[0][index].CRITERIOS.map((criterios: IDictionary) => criterios.NRCRITERIOTAREFA);
      const nrTags = dataPai[0][index].TAGS.map((tags: IDictionary) => tags.NRTAGSTAREFA);
      const tarefas = {
        NRATIVIDADE: linha!.name.split('_')[1],
        NRTAREFA: dataPai[0][index].NRTAREFA,
        NRCRITERIOTAREFA: nrCriteriosTarefas,
        NRTAGSTAREFA: nrTags,
      };
      Http.post('/validacaoTarefas', tarefas).then((response) => {
        const { validacaoTarefa } = response.data.dataset;
        const retorno = validacaoTarefa[0].VALIDA;
        if (!retorno) {
          Http.post('/deletaTarefa', { tarefas }).then(() => {
            this.renderizaCardAtividadesTeste();
          });
          linha!.children.splice(Number(variavel1), 5);
        } else {
          AlertService.show({
            name: 'basic-alert',
            text: 'Exclusão não permitida! O registro esta sendo utilizado',
            top: true,
            left: false,
            timeout: 3000,
            color: 'error',
          });
        }
        const verificarFilhos = linha!.children.map((dados: IDictionary) => dados.name);
        if (verificarFilhos.length === 5) {
          Http.post('/deletaAtividade', { tarefas });
          // Promise.all([window.location.reload()]).then(() => {               ******* Retirado para teste **********
          //   Metadata.getInstance<Tabs>('tabsLocais').activeTab = 1;
          // });
        }
      });
    }
  }

  // Função não esta sendo utilizada
  public deletaCriterioModal({ component }:{ component:Button }) {
    const linha = component!.parent!.parent;
    const variavel = linha!.children.map((dados: IDictionary) => dados.name);
    const variavel1 = variavel.indexOf(`DadosTarefas${component!.parent!.name.substr(12)}`);
    linha!.children.splice(Number(variavel1), 5);
  }

  public global: any = [];

  // função responsavel por pegar o conteudo da DSPOPSERVIÇO
  public getDescricao({ component }:{ component:Button }) {
    const selectObjectLocal = component!.parent!;
    this.global = selectObjectLocal.name.substr(12);
    this.modalController1.openHeaderFooterModal();
  }

  // Função responsavel por salvar o conteudo no TextImput que esta invisivel
  public salvaDescricao() {
    const switchObject = Metadata.getInstance<Switch>('switch');
    const selectObjectLinha = Metadata.getInstance<Textarea>(`textareaResultModal${this.global}`);
    if (switchObject.value) {
      selectObjectLinha.value = Metadata.getInstance<Textarea>('textareaResult').value;
    } else {
      selectObjectLinha.value = Metadata.getInstance<FileInput>('fileResult').storePath;
    }
    Metadata.getInstance<Textarea>('textareaResult').value = '';
    this.modalController1.closeHeaderFooterModal();
  }

  // função responsavel por pegar a informação do imput do Modal e alterar
  public editDescicao({ component }: IEventParam<Textarea>) {
    const selectObjectText = Metadata.getInstance<Textarea>(`textareaResultModal${this.global}`);
    if (selectObjectText.value != null) {
      const variavel = component!.parent!.parent!.children[1];
      variavel.value = true;
      component.value = selectObjectText.value;
    }
  }

  // Função para expandir e editar o card Atividades
  public editarCardAtividades({ component }:{ component:Button }) {
    const a = component.parent as IDictionary;
    if (!a.expanded) {
      this.createFromTypescriptModal(component.name.split('_')[1]);
      a.header[0].children[2].isVisible = !a.expanded;
      this.informacoesTarefas(component.name.split('_')[1]);
    } else {
      this.destroyFromTypescriptModal();
      a.header[0].children[2].isVisible = !a.expanded;
      const form = Metadata.getInstance<Form>(`colunaTab2teste_${a.name.split('_')[1]}`);
      form.children.splice(0, form.children.length - 5);
    }
    a.expanded = !a.expanded;
  }

  // Função responsável por adicionar nova tarefa
  public adicionaNovaTArefaCardAtividades({ component }: IEventParam<Button>) {
    const selectFormObject = component!.parent!.parent! as IDictionary;
    const selectObjectBotao = component!.parent!.name.substr(12);
    const selectObjectTarefa = Metadata.getInstance<TextInput>(`DadosTarefas${selectObjectBotao}`);
    const selectObjectCriterio = Metadata.getInstance<TextInput>(`DadosCriterio${selectObjectBotao}`);
    const selectObjectTag = Metadata.getInstance<TextInput>(`DadosTag${selectObjectBotao}`);
    const selectObjectDescricao = Metadata.getInstance<Textarea>(`textareaResultModal${selectObjectBotao}`);
    if (selectFormObject.validate()) {
      this.criaModeloLinha(selectObjectTarefa.value, selectObjectCriterio.value, selectObjectTag.value, selectObjectDescricao.value, this.contador);
      selectFormObject.children.splice(selectFormObject.children.length - 5, 0, this.modeloCriterio[0]);
      selectFormObject.children.splice(selectFormObject.children.length - 5, 0, this.modeloCriterio[1]);
      selectFormObject.children.splice(selectFormObject.children.length - 5, 0, this.modeloCriterio[2]);
      selectFormObject.children.splice(selectFormObject.children.length - 5, 0, this.modeloCriterio[3]);
      selectFormObject.children.splice(selectFormObject.children.length - 5, 0, this.modeloCriterio[4]);
      this.contador += 1;
      const dadosLinhaCardAtividades = {
        NRATIVIDADE: selectFormObject.name.split('_')[1],
        NMTAREFA: selectFormObject.value[`DadosTarefas${selectObjectBotao}`],
        CRITERIO: selectFormObject.value[`DadosCriterio${selectObjectBotao}`].replace(';', ',').split(','),
        TAG: selectFormObject.value[`DadosTag${selectObjectBotao}`].replace(';', ',').split(','),
        DESCRICAO: selectFormObject.value[`textareaResultModal${selectObjectBotao}`],
      };
      Http.post('/addNovaTarefa', { DADOS: dadosLinhaCardAtividades, NRORG: 20 }).then(() => {
        this.renderizaCardAtividadesTeste();
      });
      AlertService.show({
        name: 'basic-alert',
        text: 'Nova Tarefa adicionada com sucesso!',
        top: true,
        left: false,
        timeout: 3000,
        color: 'success',
      });
      selectObjectTarefa.value = '';
      selectObjectCriterio.value = '';
      selectObjectTag.value = '';
      selectObjectDescricao.value = '';
    }
  }

  // funcao para adicionar nova linha no card já montado na tela
  public adcionaLinhaCardAtividades({ component }: IEventParam<Form>) {
    this.criaModeloLinhaFixo('', '', '', '', this.contador);
    component.children.push(this.modeloCriterio[0]);
    component.children.push(this.modeloCriterio[1]);
    component.children.push(this.modeloCriterio[2]);
    component.children.push(this.criaFooter(this.contador)[0]);
    component.children.push(this.modeloCriterio[4]);
    this.contador += 1;
  }

  public selecionaGrupoCriterio({ component }:{ component:Chip }) {
    component.outlined = !component.outlined;
    const conjunto = component.name.split('_');
    if (!component.outlined) {
      if (Object.keys(this.dadosChip).includes(conjunto[1])) {
        if (Object.keys(this.dadosChip[conjunto[1]]).includes(String(conjunto[2]))) {
          this.dadosChip[conjunto[1]][conjunto[2]].push(Number(conjunto[3]));
        } else {
          this.dadosChip[conjunto[1]][conjunto[2]] = [];
          this.dadosChip[conjunto[1]][conjunto[2]].push(Number(conjunto[3]));
        }
      } else {
        this.dadosChip[conjunto[1]] = {};
        this.dadosChip[conjunto[1]][conjunto[2]] = [];
        this.dadosChip[conjunto[1]][conjunto[2]].push(Number(conjunto[3]));
      }
    } else {
      this.dadosChip[conjunto[1]][conjunto[2]].splice(this.dadosChip[conjunto[1]][conjunto[2]].indexOf(Number(conjunto[3])), 1);
      if (!this.dadosChip[conjunto[1]][conjunto[2]].length) {
        delete this.dadosChip[conjunto[1]][conjunto[2]];
      }
      if (!Object.keys(this.dadosChip[conjunto[1]]).length) {
        delete this.dadosChip[conjunto[1]];
      }
    }
  }

  // funcao para o campo de busca card nos tabs
  public busca() {
    const selectObjectTab = Metadata.getInstance<Tabs>('tabsLocais');
    const searchObject = Metadata.getInstance<TextInput>('campoPesquisar');
    if (selectObjectTab.activeTab === 0) {
      this.dataFilter.inputLocal = searchObject.value;
      this.renderizaCard();
    } else {
      this.dataFilter.inputAtividades = searchObject.value;
      this.renderizaCardAtividadesTeste();
    }
  }

  public async getEditDados() {
    return new Promise((resolve, reject) => {
      try {
        const routeParams = getRouteParams();
        const { nr } = routeParams;
        Http.post('/getDadosEditCard', { nr }).then((response) => {
          const { dados } = response.data.dataset;
          Metadata.getInstance<Select>('selectLocal').setValue(dados.NRTIPOLOCALCLI);
          Metadata.getInstance<TextInput>('nomeCheckList').value = dados.NMCHECKLIST;
          Metadata.getInstance<TextInput>('descricaoChecklist').value = dados.DSCHECKLIST;
          const selectItens = dados.ITEM.map((valor: IDictionary) => Number(valor.NRATIVIDADE));
          Metadata.getInstance<SelectMultiple>('selectItens').setValue(selectItens);
          let teste = dados.ITEM.map((valor: IDictionary) => valor.TAREFA);
          teste = teste.filter((valor: IDictionary) => valor);
          const variavel = teste.map((dados1: IDictionary) => dados1.map((dadoTarefa: IDictionary) => dadoTarefa.NRTAREFA));
          let tarefas: IDictionary = [];
          variavel.forEach((item: IDictionary) => {
            tarefas = tarefas.concat(item);
          });
          const valueTarefa: IDictionary = [];
          tarefas.forEach((valor: number) => {
            valueTarefa.push({ value: valor });
          });
          const tags = teste.map((tag: IDictionary) => tag.map((dadosTag: IDictionary) => dadosTag.TAGS));
          const grupoTag = tags.flat(2).filter((tag: IDictionary) => tag);
          const selectChip = teste.map((tag: IDictionary) => tag.map((dadosTag: IDictionary) => dadosTag.RELACCRITTAG));
          const grupoChip = selectChip.flat(2).filter((tag: IDictionary) => tag);

          grupoChip.forEach((dadosGrupoChip: IDictionary) => {
            const tipo = grupoTag.filter((tag: IDictionary) => tag.NRTAGSCHKTAREFA === dadosGrupoChip.NRTAGSCHKTAREFA);
            dadosGrupoChip.tipo = tipo[0].DSTAGSTAREFA;
          });

          resolve({
            tarefa: valueTarefa,
            chip: grupoChip,
          });
        });
      } catch (error) {
        reject(error);
      }
      const routeParams = getRouteParams();
      const { nr } = routeParams;
      Http.post('/validacao', { nr }).then((response) => {
        const { validacao } = response.data.dataset;
        const retorno = validacao[0].VALIDA;
        if (!retorno) {
          Metadata.getInstance<Select>('selectLocal').disabled = true;
        } else {
          Metadata.getInstance<Select>('selectLocal').disabled = true;
          // Metadata.getInstance<TextInput>('nomeCheckList').disabled = true;
          // Metadata.getInstance<TextInput>('descricaoChecklist').disabled = true;
          Metadata.getInstance<SelectMultiple>('selectItens').disabled = true;
          Metadata.getInstance<SelectMultiple>('selectTagTarefas').disabled = true;
          Metadata.getInstance<Button>('botaoCriar').disabled = true;
          AlertService.show({
            name: 'basic-alert',
            text: 'Não é possivel alterar essa checklist',
            top: true,
            left: false,
            timeout: 3000,
            color: 'error',
          });
        }
      });
    });
  }

  public updateChecklist() {
    const selectObjectForm = Metadata.getInstance<Form>('FormPrincipal');
    if (selectObjectForm.validate()) {
      if (Object.keys(this.dadosChip).length) {
        const routeParams = getRouteParams();
        const { nr } = routeParams;
        const arrayDados = {
          tipoLocal: Metadata.getInstance<Select>('selectLocal').value,
          nomeCheckList: Metadata.getInstance<TextInput>('nomeCheckList').value,
          descricaoChecklist: Metadata.getInstance<TextInput>('descricaoChecklist').value,
          itensChecklist: Metadata.getInstance<SelectMultiple>('selectItens').value,
          selectTagTarefas: Metadata.getInstance<SelectMultiple>('selectTagTarefas').value,
          dadosChip: this.dadosChip,
        };
        DialogService.show({
          name: 'basic-dialog',
          title: 'Atualização de CheckList',
          text: 'Aguarde......',
          buttons: [
            {
              name: 'yes',
              component: 'ZdButton',
              label: 'Sair',
              isVisible: false,
            },
          ],
        });
        Http.post('/updateChecklist', { DADOS: arrayDados, NRORG: 20, NRCHECKLIST: nr }).then((response) => {
          const atualizado = response;
          if (atualizado.statusText === 'OK') {
            DialogService.hide();
            Router.replace('/checklist');
          }
        });
      } else {
        AlertService.show({
          name: 'basic-alert',
          text: 'Favor preencher pelo menos um critério',
          top: true,
          left: false,
          timeout: 3000,
          color: 'error',
        });
      }
    }
  }

  public deleteCard({ component }: IEventParam<Button>) {
    const nr = component.name.split('_')[1];
    Http.post('/validacao', { nr }).then((response) => {
      const { validacao } = response.data.dataset;
      const retorno = validacao[0].VALIDA;
      if (!retorno) {
        Http.post('/deleteChecklist', { NRCHECKLIST: nr });
        AlertService.show({
          name: 'basic-alert',
          text: 'CheckList excluida com sucesso!',
          top: true,
          left: false,
          timeout: 3000,
          color: 'success',
        });
        setTimeout(() => {
          this.renderizaCard();
        }, 1000);
      } else {
        AlertService.show({
          name: 'basic-alert',
          text: 'Operação bloqueada! Não é possivel excluir essa checklist, pois a mesma esta em uso.',
          top: true,
          left: false,
          timeout: 3000,
          color: 'error',
        });
      }
    });
  }

  public alterarCorChip({ component }:{ component:Chip }) {
    const status = (String(component.reverse) === 'false');
    component.outlined = status;
  }

  public async createFromTypescriptModal(id: string) {
    this.fromTypescriptModal = ModalService.create({
      name: `fromTypescriptModalr_${id}`,
      persistent: true,
      grid: {
        lg: 3,
        md: 4,
        sm: 5,
      },
      cssClass: 'Test123456',
      children: [
        {
          name: `SelectedColor_${id}`,
          component: 'ZdColorPicker',
          value: '#FF0000',
        },
        {
          name: 'RowWithButtonOfModal',
          component: 'ZdRow',
          style: {
            width: '100%',
            'max-width': '300px',
            'justify-content': 'center',
            margin: '10px 5px',
          },
          children: [
            {
              name: 'CancelarBtn',
              component: 'ZdButton',
              label: 'Cancelar',
              outline: true,
              style: {
                width: '100%',
                'max-width': '135px',
                'margin-right': '5px',
              },
              events: {
                click: () => this.fromTypescriptModal.hide(),
              },
            },
            {
              name: `botaoConfirmaColor_${id}`,
              component: 'ZdButton',
              label: 'Confirmar',
              style: {
                width: '100%',
                'max-width': '135px',
                'margin-left': '5px',
              },
              events: {
                click: ({ component }:{ component:any }) => this.closeFromTypescriptModal(component),
              },
            },
          ],
        },
      ],
    });
  }

  public destroyFromTypescriptModal() {
    this.fromTypescriptModal?.destroy();
  }

  public async openFromTypescriptModal({ component }:{ component:Button }) {
    const id = component.name.split('_')[1];
    await this.fromTypescriptModal.show();
    const colorP = Metadata.getInstance<ZdColorPicker>(`SelectedColor_${id}`);
    const icon = Metadata.getInstance<Icon>(`SquareColor_${id}`);
    colorP.value = icon.color;
  }

  public closeFromTypescriptModal(component: any) {
    const id = component.name.split('_')[1];
    const cor = Metadata.getInstance<ZdColorPicker>(`SelectedColor_${id}`).value;
    Http.get(`/changeColorActivity?nr=${id}&color=${String(cor).substr(1)}`).then((response: IDictionary) => {
      /* eslint-disable no-console */
      if (response.data.dataset.dados[0] === 'OK') {
        AlertService.show({
          name: 'basic-alert',
          text: 'Cor atualizada com sucesso!',
          top: true,
          left: false,
          timeout: 3000,
          color: 'success',
        });
        Metadata.getInstance<Icon>(`SquareColor_${id}`).color = cor as string;
      } else {
        DialogService.show({
          name: 'basic-dialog',
          type: 'warning',
          title: 'Transação bloqueada!',
          text: 'Ocorreu um erro durante a transferencia.\nTente novamente mais tarde.\nSe o erro persistir, entre em contato com o administrador do sistema.',
          buttons: [
            {
              name: 'cancel',
              component: 'ZdButton',
              outline: true,
              label: 'Voltar',
              events: {
                click: () => DialogService.hide(),
              },
            },
          ],
        });
      }
    }).then(() => {
      this.fromTypescriptModal.hide();
    });
  }
}
