/* eslint-disable no-console */
import {
  Button, Col, Form, IModal, Modal, ModalService, Select, TextInput, AlertService, DialogService, Row,
} from '@zeedhi/common';
import { IDictionary, Http, Metadata } from '@zeedhi/core';

export class ParametrosSensorController {
  public mapAlertModal: IDictionary = {};

  public SensorModal!: Modal;

  public UpdateSensorModal!: Modal;

  public Sensors: IDictionary = {};

  public Clients: Array<IDictionary> = [];

  public Locais: Array<IDictionary> = [];

  public localId: number = 0;

  public TypeSensors: Array<IDictionary> = [];

  public DistanceMode: Array<IDictionary> = [];

  public disabledSensorFields: boolean = true;

  public CardCounter: number = 0;

  public listSensor() {
    const rowObject = Metadata.getInstance<Row>('row2');
    rowObject.children = []; // após atualizar alguma coisa pode chamar essa função e dar um reload no children do component com os novos dados
    Http.get('/ParamLeitContPessoas')
      .then((response: IDictionary) => {
        const sensors = response.data.dataset.Sensores;
        sensors.map((sensor: IDictionary, index: number) => {
          const card = this.newCard(index.toString(), sensor.IDMACDEVICE, sensor.CDCLIENTE, sensor.NRPARAMLEITCONTPESSOAS.toString());
          rowObject.children.push(card);
          this.CardCounter += 1;
          return card;
        });
      });
  }

  public listLocais({ component }: { component: Select }) {
    const { value } = component;
    Http.get(`/Locals?CDCLIENTE=${value}`)
      .then((response: IDictionary) => {
        const selectLocal: Select = Metadata.getInstance('NRLOCALCLIENTE');
        this.Locais = response.data.dataset.Locals;
        selectLocal.datasource.data = this.Locais;
        selectLocal.setValue(this.localId);
      });
  }

  public listClients({ component }: { component: Select }) {
    Http.get('/Clients/simpleView')
      .then((response: IDictionary) => {
        this.Clients = response.data.dataset.Clientes;
        component.datasource.data = this.Clients;
      });
  }

  public listTypeSensors({ component }: { component: Select }) {
    const fixed = [
      {
        id: 1,
        name: 'Infravermelho',
      },
      {
        id: 2,
        name: 'Distância',
      },
      {
        id: 3,
        name: 'Câmera',
      },
    ];
    component.datasource.updateData(fixed);
  }

  public listDistanceMode({ component }: { component: Select }) {
    const fixedDistances = [
      {
        id: 1,
        name: 'Curta',
      },
      {
        id: 2,
        name: 'Longa',
      },
    ];
    component.datasource.updateData(fixedDistances);
  }

  public listAlertTypes({ component } : { component: Select }) {
    const fixedAlertTypes = [
      {
        id: 1,
        name: 'Email',
      },
    ];

    component.datasource.updateData(fixedAlertTypes);
  }

  public newCard(v1: string, macAddress: string, client: string, idSensor: string) {
    return {
      name: `col2-${v1}`,
      component: 'ZdCol',
      cols: 12,
      children: [
        {
          name: `card-2-${v1}`,
          component: 'ZdCard',
          hover: true,
          elevation: 5,
          children: [
            {
              name: `insideRow2-${v1}`,
              component: 'ZdRow',
              children: [
                {
                  name: `insideCol-2-${v1}-1`,
                  component: 'ZdCol',
                  cols: 4,
                  align: 'right',
                  children: [
                    {
                      name: 'RowCient',
                      component: 'ZdRow',
                      align: 'center',
                      children: [
                        {
                          name: `icon-2-${v1}-1`,
                          component: 'ZdButton',
                          iconName: 'mdi-account',
                          icon: true,
                          small: true,
                        },
                        {
                          name: `text-2-${v1}-2`,
                          component: 'ZdText',
                          cssClass: 'zd-pl-3',
                          text: client,
                        },
                      ],
                    },
                  ],
                },
                {
                  name: `insideCol-2-${v1}-3`,
                  component: 'ZdCol',
                  cols: 4,
                  align: 'right',
                  children: [
                    {
                      name: 'RowMACAdress',
                      component: 'ZdRow',
                      align: 'center',
                      children: [
                        {
                          name: `icon2-${v1}-3`,
                          component: 'ZdButton',
                          iconName: 'mdi-desktop-mac',
                          icon: true,
                          small: true,
                        },
                        {
                          name: `text-2-${v1}-4`,
                          component: 'ZdText',
                          cssClass: 'zd-pl-3',
                          text: macAddress,
                        },
                      ],
                    },
                  ],
                },
                {
                  name: `insideCol-2-${v1}-5`,
                  component: 'ZdCol',
                  cols: 4,
                  align: 'right',
                  children: [
                    {
                      name: `update_${idSensor}`,
                      component: 'ZdButton',
                      icon: true,
                      iconName: 'mdi-pencil',
                      small: true,
                      events: {
                        click: this.openUpdateSensorModal.bind(this),
                      },
                    },
                    {
                      name: `${idSensor}`,
                      component: 'ZdButton',
                      icon: true,
                      iconName: 'mdi-bell',
                      small: true,
                      events: {
                        onCreated: this.createAlertModal.bind(this),
                        onBeforeDestroy: this.destroyAlertModal.bind(this),
                        click: this.openAlertModal.bind(this),
                      },
                    },
                  ],
                },
              ],
            },
          ],
        },
      ],
    };
  }

  public getSensorExclusiveFields() {
    const elements = [];
    const distMin: TextInput = Metadata.getInstance('VRDISTANCIAMINLEIT');
    const distMax: TextInput = Metadata.getInstance('VRDISTANCIAMAXLEIT');
    const distThresh: TextInput = Metadata.getInstance('VRDISTANCIAMEDLEIT');
    const rowsOfSpad: TextInput = Metadata.getInstance('QTMATRIZLEITURA');
    const timmingBudget: TextInput = Metadata.getInstance('QTTEMPRECONHECIMENTO');
    const distanceMode: Select = Metadata.getInstance('IDMODODISTANCIA');
    elements.push(distMin);
    elements.push(distMax);
    elements.push(distThresh);
    elements.push(rowsOfSpad);
    elements.push(timmingBudget);
    elements.push(distanceMode);
    return elements;
  }

  public getGeneralFields() {
    const elements = [];
    const codCliente: Select = Metadata.getInstance('CDCLIENTE');
    const localCliente: Select = Metadata.getInstance('NRLOCALCLIENTE');
    const idTipoSensor: Select = Metadata.getInstance('IDTIPOSENSOR');
    const macAddress: TextInput = Metadata.getInstance('IDMACDEVICE');
    const idSensor: TextInput = Metadata.getInstance('IDSENSOR');
    const quantPeopleAlert: TextInput = Metadata.getInstance('NRQTPESSOASALERT');
    const hourInit: TextInput = Metadata.getInstance('HRINICIODIA');
    const hourEnd: TextInput = Metadata.getInstance('HRFINALDIA');
    const intervalRequests: TextInput = Metadata.getInstance('QTTEMPOREPETLEIT');
    elements.push(codCliente);
    elements.push(localCliente);
    elements.push(idTipoSensor);
    elements.push(macAddress);
    elements.push(idSensor);
    elements.push(quantPeopleAlert);
    elements.push(hourInit);
    elements.push(hourEnd);
    elements.push(intervalRequests);
    return elements;
  }

  public changeDisableFields({ component }: { component: Select }) {
    const { value } = component;
    // apenas inverte o valor do atributo disabled no componente
    if ((value === 2 && this.disabledSensorFields) || (value !== 2 && !this.disabledSensorFields)) {
      const fields = this.getSensorExclusiveFields();
      this.disabledSensorFields = !this.disabledSensorFields;
      fields.map((item) => {
        item.disabled = this.disabledSensorFields;
        return 0;
      });
    }
  }

  public createSensorModal() {
    const sensorModal: IModal = {
      name: 'sensorModal',
      persistent: true,
      fullscreen: false,
      children: [
        {
          name: 'modalHeader',
          component: 'ZdHeader',
          color: 'transparent',
          padless: true,
          elevation: 0,
          leftSlot: [
            {
              name: 'colTitleText',
              component: 'ZdCol',
              align: 'center',
              cols: 12,
              children: [
                {
                  name: 'titleText',
                  component: 'ZdText',
                  text: 'Novo sensor',
                  cssClass: 'title-modal',
                },
              ],
            },
          ],
          rightSlot: [
            {
              name: 'closeModalButton',
              component: 'ZdButton',
              icon: true,
              iconName: 'mdi-close',
              events: {
                click: this.closeSensorModal.bind(this),
              },
            },
          ],
        },
        {
          name: 'sensorForm',
          component: 'ZdForm',
          children: [
            {
              name: 'row4',
              component: 'ZdRow',
              children: [
                {
                  name: 'col-4-1',
                  component: 'ZdCol',
                  cols: 6,
                  children: [
                    {
                      name: 'CDCLIENTE',
                      component: 'ZdSelect',
                      autocomplete: false,
                      datasource: {
                        loadAll: true,
                        data: [],
                      },
                      dataText: 'NMRAZSOCCLIE',
                      dataValue: 'CDCLIENTE',
                      label: 'Cliente',
                      events: {
                        onCreated: this.listClients.bind(this),
                        change: this.listLocais.bind(this),
                      },
                      validations: {
                        required: {},
                      },
                    },
                  ],
                },
                {
                  name: 'col-4-2',
                  component: 'ZdCol',
                  cols: 6,
                  children: [
                    {
                      name: 'NRLOCALCLIENTE',
                      component: 'ZdSelect',
                      autocomplete: false,
                      datasource: {
                        loadAll: true,
                        data: [],
                      },
                      dataText: 'NMLOCALCLIENTE',
                      dataValue: 'NRLOCALCLIENTE',
                      label: 'Local',
                      validations: {
                        required: {},
                      },
                    },
                  ],
                },
                {
                  name: 'col-4-3',
                  component: 'ZdCol',
                  cols: 6,
                  children: [
                    {
                      name: 'IDTIPOSENSOR',
                      component: 'ZdSelect',
                      autocomplete: false,
                      datasource: {
                        data: [],
                      },
                      dataValue: 'id',
                      dataText: 'name',
                      label: 'Tipo de sensor',
                      events: {
                        onCreated: this.listTypeSensors.bind(this),
                        change: this.changeDisableFields.bind(this),
                      },
                      validations: {
                        required: {},
                      },
                    },
                  ],
                },
                {
                  name: 'col-4-4',
                  component: 'ZdCol',
                  cols: 6,
                  children: [
                    {
                      name: 'IDMACDEVICE',
                      component: 'ZdTextInput',
                      label: 'MAC Address do Microcontrolador',
                      validations: {
                        required: {},
                      },
                    },
                  ],
                },
                {
                  name: 'col-4-5',
                  component: 'ZdCol',
                  cols: 6,
                  children: [
                    {
                      name: 'IDSENSOR',
                      component: 'ZdTextInput',
                      label: 'ID Sensor',
                      validations: {
                        required: {},
                      },
                    },
                  ],
                },
                {
                  name: 'col-4-6',
                  component: 'ZdCol',
                  cols: 6,
                  children: [
                    {
                      name: 'NRQTPESSOASALERT',
                      component: 'ZdTextInput',
                      label: 'Número de pessoas para enviar o alerta',
                      type: 'number',
                      validations: {
                        required: {},
                      },
                    },
                  ],
                },
                {
                  name: 'col-4-14',
                  component: 'ZdCol',
                  cols: 6,
                  children: [
                    {
                      name: 'HRINICIODIA',
                      component: 'ZdTime',
                      timeFormat: '24hr',
                      label: 'Horário de inicio da contagem',
                      validations: {
                        required: {},
                      },
                    },
                  ],
                },
                {
                  name: 'col-4-15',
                  component: 'ZdCol',
                  cols: 6,
                  children: [
                    {
                      name: 'HRFINALDIA',
                      component: 'ZdTime',
                      timeFormat: '24hr',
                      label: 'Horário de fim da contagem',
                      validations: {
                        required: {},
                      },
                    },
                  ],
                },
                {
                  name: 'col-4-13',
                  component: 'ZdCol',
                  cols: 12,
                  children: [
                    {
                      name: 'QTTEMPOREPETLEIT',
                      component: 'ZdTextInput',
                      label: 'Intervalo entre os registros (segundos)',
                      type: 'number',
                      validations: {
                        required: {},
                      },
                    },
                  ],
                },
                {
                  name: 'col-4-7',
                  component: 'ZdCol',
                  cols: 6,
                  children: [
                    {
                      name: 'VRDISTANCIAMINLEIT',
                      component: 'ZdTextInput',
                      label: 'Distância minima (mm)',
                      type: 'number',
                      disabled: this.disabledSensorFields,
                    },
                  ],
                },
                {
                  name: 'col-4-8',
                  component: 'ZdCol',
                  cols: 6,
                  children: [
                    {
                      name: 'VRDISTANCIAMAXLEIT',
                      component: 'ZdTextInput',
                      label: 'Distância máxima (mm)',
                      type: 'number',
                      disabled: this.disabledSensorFields,
                    },
                  ],
                },
                {
                  name: 'col-4-9',
                  component: 'ZdCol',
                  cols: 6,
                  children: [
                    {
                      name: 'VRDISTANCIAMEDLEIT',
                      component: 'ZdTextInput',
                      label: 'Distância limite (mm)',
                      type: 'number',
                      disabled: this.disabledSensorFields,
                    },
                  ],
                },
                {
                  name: 'col-4-10',
                  component: 'ZdCol',
                  cols: 6,
                  children: [
                    {
                      name: 'QTMATRIZLEITURA',
                      component: 'ZdTextInput',
                      label: 'Numero de linhas da matriz',
                      type: 'number',
                      disabled: this.disabledSensorFields,
                    },
                  ],
                },
                {
                  name: 'col-4-11',
                  component: 'ZdCol',
                  cols: 6,
                  children: [
                    {
                      name: 'QTTEMPRECONHECIMENTO',
                      component: 'ZdTextInput',
                      label: 'Tempo de leitura (ms)',
                      type: 'number',
                      disabled: this.disabledSensorFields,
                    },
                  ],
                },
                {
                  name: 'col-4-12',
                  component: 'ZdCol',
                  cols: 6,
                  children: [
                    {
                      name: 'IDMODODISTANCIA',
                      component: 'ZdSelect',
                      autocomplete: false,
                      label: 'Modo de distância',
                      datasource: {
                        data: [],
                      },
                      dataText: 'name',
                      dataValue: 'id',
                      events: {
                        onCreated: this.listDistanceMode.bind(this),
                      },
                      disabled: this.disabledSensorFields,
                    },
                  ],
                },
              ],
            },
          ],
        },
        {
          name: 'footer',
          component: 'ZdFooter',
          color: 'transparent',
          padless: true,
          rightSlot: [
            {
              name: 'cancel',
              component: 'ZdButton',
              label: 'Cancelar',
              outline: true,
              events: {
                click: this.closeSensorModal.bind(this),
              },
            },
            {
              name: 'save',
              component: 'ZdButton',
              label: 'Salvar',
              cssClass: 'zd-px-7',
              events: {
                click: this.insertSensor.bind(this),
              },
            },
          ],
        },
      ],
    };
    this.SensorModal = ModalService.create(sensorModal);
  }

  public destroySensorModal() {
    this.SensorModal.destroy();
  }

  public openSensorModal() {
    this.localId = 0;
    this.createSensorModal();
    this.SensorModal.show();
  }

  public closeSensorModal() {
    this.SensorModal.destroy();
    this.SensorModal.hide();
  }

  public createAlertModal({ component }: { component: Button }) {
    const idSensor = component.name;
    const aModal: IModal = {
      name: `alertModal-${idSensor}`,
      component: 'ZdModal',
      persistent: false,
      fullscreen: false,
      style: {
        background: '#black',
      },
      children: [
        {
          name: 'modalHeader',
          component: 'ZdHeader',
          color: 'transparent',
          padless: true,
          elevation: 0,
          leftSlot: [
            {
              name: 'titleText',
              component: 'ZdText',
              text: 'Alertas',
              cssClass: 'title-modal',
            },
          ],
          rightSlot: [
            {
              name: 'closeModalButton',
              component: 'ZdModalCloseButton',
              modalName: `alertModal-${idSensor}`,
            },
          ],
        },
        {
          name: 'form-alert',
          component: 'ZdForm',
          cssClass: 'zd-px-2',
          children: [
            {
              name: 'typeAlert',
              component: 'ZdSelect',
              datasource: {
                loadAll: true,
                data: [],
              },
              dataValue: 'id',
              dataText: 'name',
              label: 'Tipo de alerta',
              events: {
                onCreated: this.listAlertTypes.bind(this),
              },
              validations: {
                required: {},
              },
              grid: {
                cols: 12,
              },
            },
            {
              name: 'contactAlert',
              component: 'ZdTextInput',
              label: 'Contato para o alerta',
              validations: {
                required: {},
              },
              grid: {
                cols: 12,
              },
            },
          ],
        },
        {
          name: 'alertFooter',
          component: 'ZdFooter',
          color: 'transparent',
          padless: true,
          rightSlot: [
            {
              name: idSensor,
              component: 'ZdButton',
              label: 'Cancelar',
              outline: true,
              events: {
                click: this.closeAlertModal.bind(this),
              },
            },
            {
              name: `btn-${idSensor}`,
              component: 'ZdButton',
              label: 'Salvar',
              cssClass: 'zd-px-7',
              events: {
                click: this.createNewAlert.bind(this),
              },
            },
          ],
        },
      ],
    };
    const AlertModal: IModal = ModalService.create(aModal);
    this.mapAlertModal[idSensor] = AlertModal;
  }

  public createNewAlert({ component }: { component: Button }) {
    const formAlert: Form = Metadata.getInstance('form-alert');
    const validate = formAlert.validate();
    if (validate) {
      const splitedName = component.name.split('-');
      const idSensor = splitedName[1];
      const contactField: TextInput = Metadata.getInstance('contactAlert');
      const typeAlertField: Select = Metadata.getInstance('typeAlert');
      const bodyPost = {
        NRPARAMLEITCONTPESSOAS: parseInt(idSensor, 10),
        DSCONTATOSALERTA: contactField.value,
        IDTIPOALERTA: typeAlertField.value,
        NRORG: 20,
      };
      Http.post('/ParamAlertContPessoas', bodyPost)
        .then(() => {
          AlertService.show({
            name: 'basic-alert',
            text: `Novo alerta inserido com sucesso para o sensor ${idSensor}.`,
            top: true,
            left: false,
            timeout: 3000,
            color: 'success',
          });
          typeAlertField.value = '';
        })
        .catch((err) => {
          AlertService.show({
            name: 'basic-alert',
            text: err,
            top: true,
            left: false,
            timeout: 3000,
            color: 'error',
          });
        });

      const macAddress = component.name.substr(4); // Fechar o modal após inserir alerta
      this.mapAlertModal[macAddress].hide();
    }
  }

  public destroyAlertModal({ component }: { component: Button }) {
    const macAddress = component.name;
    this.mapAlertModal[macAddress].destroy();
  }

  public openAlertModal({ component }: { component: Button }) {
    const macAddress = component.name;
    this.mapAlertModal[macAddress].show();
  }

  public closeAlertModal({ component }: { component: Button }) {
    const macAddress = component.name;
    this.mapAlertModal[macAddress].hide();
  }

  public insertSensor() {
    const form: Form = Metadata.getInstance('sensorForm');
    const validate = form.validate();
    if (validate) {
      const bodyPost: IDictionary = {};
      let fields = this.getGeneralFields();
      const sensorFields = this.getSensorExclusiveFields();
      if (!this.disabledSensorFields) {
        fields = fields.concat(sensorFields);
      } else {
        sensorFields.forEach((item) => {
          const nameValue: string = item.name;
          bodyPost[nameValue] = 0;
        });
      }
      fields.forEach((field) => {
        const nameValue: string = field.name;
        bodyPost[nameValue] = Number.isNaN(field.value) ? parseFloat(field.value) : field.value;
      });
      Http.post('/ParamLeitContPessoas', bodyPost)
        .then((response) => {
          const numInserted = response.data.dataset.ItemID[0];
          const row: Col = Metadata.getInstance('row2');
          const newCard = this.newCard(this.CardCounter.toString(), bodyPost.IDMACDEVICE, bodyPost.CDCLIENTE, numInserted.toString());
          row.children.push(newCard);
          AlertService.show({
            name: 'basic-alert',
            text: 'Novos parametros de sensor foram criados com sucesso.',
            top: true,
            left: false,
            timeout: 3000,
            color: 'success',
          });
          fields.forEach((field) => {
            field.value = '';
          });
        }).catch((err) => {
          AlertService.show({
            name: 'basic-alert',
            text: err,
            top: true,
            left: false,
            timeout: 3000,
            color: 'success',
          });
        });
    }
  }

  /* --------- Tratamento do modal de editar Sensores --------- */

  public createUpdateSensorModal(sensorId: string) {
    const updateSensorModal: IModal = {
      name: 'updateSensorModal',
      persistent: true,
      fullscreen: false,
      style: {
        background: '#black',
      },
      children: [
        {
          name: 'modalHeader',
          component: 'ZdHeader',
          color: 'transparent',
          padless: true,
          elevation: 0,
          leftSlot: [
            {
              name: 'colTitleText',
              component: 'ZdCol',
              align: 'center',
              cols: 12,
              children: [
                {
                  name: 'updateTitle',
                  component: 'ZdText',
                  text: 'Editar sensor',
                  cssClass: 'title-modal',
                },
              ],
            },
          ],
        },
        {
          name: 'updateSensorForm',
          component: 'ZdForm',
          cssClass: 'zd-px-4',
          children: [
            {
              name: 'row4',
              component: 'ZdRow',
              children: [
                {
                  name: 'col-4-1',
                  component: 'ZdCol',
                  cols: 6,
                  children: [
                    {
                      name: 'CDCLIENTE',
                      component: 'ZdSelect',
                      autocomplete: false,
                      datasource: {
                        loadAll: true,
                        data: [],
                      },
                      dataText: 'NMRAZSOCCLIE',
                      dataValue: 'CDCLIENTE',
                      label: 'Cliente',
                      events: {
                        onCreated: this.listClients.bind(this),
                        change: this.listLocais.bind(this),
                      },
                      validations: {
                        required: {},
                      },
                    },
                  ],
                },
                {
                  name: 'col-4-2',
                  component: 'ZdCol',
                  cols: 6,
                  children: [
                    {
                      name: 'NRLOCALCLIENTE',
                      component: 'ZdSelect',
                      autocomplete: false,
                      datasource: {
                        loadAll: true,
                        data: [],
                      },
                      dataText: 'NMLOCALCLIENTE',
                      dataValue: 'NRLOCALCLIENTE',
                      label: 'Local',
                    },
                  ],
                },
                {
                  name: 'col-4-3',
                  component: 'ZdCol',
                  cols: 6,
                  children: [
                    {
                      name: 'IDTIPOSENSOR',
                      component: 'ZdSelect',
                      autocomplete: false,
                      datasource: {
                        data: [],
                      },
                      dataValue: 'id',
                      dataText: 'name',
                      label: 'Tipo de sensor',
                      events: {
                        onCreated: this.listTypeSensors.bind(this),
                        change: this.changeDisableFields.bind(this),
                      },
                      validations: {
                        required: {},
                      },
                    },
                  ],
                },
                {
                  name: 'col-4-4',
                  component: 'ZdCol',
                  cols: 6,
                  children: [
                    {
                      name: 'IDMACDEVICE',
                      component: 'ZdTextInput',
                      label: 'MAC Address do Microcontrolador',
                      validations: {
                        required: {},
                      },
                    },
                  ],
                },
                {
                  name: 'col-4-5',
                  component: 'ZdCol',
                  cols: 6,
                  children: [
                    {
                      name: 'IDSENSOR',
                      component: 'ZdTextInput',
                      label: 'ID Sensor',
                      validations: {
                        required: {},
                      },
                    },
                  ],
                },
                {
                  name: 'col-4-6',
                  component: 'ZdCol',
                  cols: 6,
                  children: [
                    {
                      name: 'NRQTPESSOASALERT',
                      component: 'ZdTextInput',
                      label: 'Número de pessoas para enviar o alerta',
                      type: 'number',
                      validations: {
                        required: {},
                      },
                    },
                  ],
                },
                {
                  name: 'col-4-14',
                  component: 'ZdCol',
                  cols: 6,
                  children: [
                    {
                      name: 'HRINICIODIA',
                      component: 'ZdTime',
                      timeFormat: '24hr',
                      label: 'Horário de inicio da contagem',
                      validations: {
                        required: {},
                      },
                    },
                  ],
                },
                {
                  name: 'col-4-15',
                  component: 'ZdCol',
                  cols: 6,
                  children: [
                    {
                      name: 'HRFINALDIA',
                      component: 'ZdTime',
                      timeFormat: '24hr',
                      label: 'Horário de fim da contagem',
                      validations: {
                        required: {},
                      },
                    },
                  ],
                },
                {
                  name: 'col-4-13',
                  component: 'ZdCol',
                  cols: 12,
                  children: [
                    {
                      name: 'QTTEMPOREPETLEIT',
                      component: 'ZdTextInput',
                      label: 'Intervalo entre os registros (segundos)',
                      type: 'number',
                      validations: {
                        required: {},
                      },
                    },
                  ],
                },
                {
                  name: 'col-4-7',
                  component: 'ZdCol',
                  cols: 6,
                  children: [
                    {
                      name: 'VRDISTANCIAMINLEIT',
                      component: 'ZdTextInput',
                      label: 'Distância minima (mm)',
                      type: 'number',
                      disabled: this.disabledSensorFields,
                    },
                  ],
                },
                {
                  name: 'col-4-8',
                  component: 'ZdCol',
                  cols: 6,
                  children: [
                    {
                      name: 'VRDISTANCIAMAXLEIT',
                      component: 'ZdTextInput',
                      label: 'Distância máxima (mm)',
                      type: 'number',
                      disabled: this.disabledSensorFields,
                    },
                  ],
                },
                {
                  name: 'col-4-9',
                  component: 'ZdCol',
                  cols: 6,
                  children: [
                    {
                      name: 'VRDISTANCIAMEDLEIT',
                      component: 'ZdTextInput',
                      label: 'Distância limite (mm)',
                      type: 'number',
                      disabled: this.disabledSensorFields,
                    },
                  ],
                },
                {
                  name: 'col-4-10',
                  component: 'ZdCol',
                  cols: 6,
                  children: [
                    {
                      name: 'QTMATRIZLEITURA',
                      component: 'ZdTextInput',
                      label: 'Numero de linhas da matriz',
                      type: 'number',
                      disabled: this.disabledSensorFields,
                    },
                  ],
                },
                {
                  name: 'col-4-11',
                  component: 'ZdCol',
                  cols: 6,
                  children: [
                    {
                      name: 'QTTEMPRECONHECIMENTO',
                      component: 'ZdTextInput',
                      label: 'Tempo de leitura (ms)',
                      type: 'number',
                      disabled: this.disabledSensorFields,
                    },
                  ],
                },
                {
                  name: 'col-4-12',
                  component: 'ZdCol',
                  cols: 6,
                  children: [
                    {
                      name: 'IDMODODISTANCIA',
                      component: 'ZdSelect',
                      autocomplete: false,
                      label: 'Modo de distância',
                      datasource: {
                        data: [],
                      },
                      dataText: 'name',
                      dataValue: 'id',
                      events: {
                        onCreated: this.listDistanceMode.bind(this),
                      },
                      disabled: this.disabledSensorFields,
                    },
                  ],
                },
              ],
            },
          ],
        },
        {
          name: `footer_${sensorId}`,
          component: 'ZdFooter',
          color: 'transparent',
          events: {
            onMounted: this.getSensorInfo.bind(this),
          },
          rightSlot: [
            {
              name: `cancel_${sensorId}`,
              component: 'ZdButton',
              label: 'Cancelar',
              outline: true,
              events: {
                click: this.destroyUpdateSensorModal.bind(this),
              },
            },
            {
              name: `save_${sensorId}`,
              component: 'ZdButton',
              label: 'Salvar',
              cssClass: 'zd-px-7',
              events: {
                click: this.updateSensor.bind(this),
              },
            },
          ],
        },
      ],
    };
    this.UpdateSensorModal = ModalService.create(updateSensorModal);
  }

  public destroyUpdateSensorModal() {
    this.UpdateSensorModal.destroy();
  }

  public openUpdateSensorModal({ component } : { component:Button }) {
    this.createUpdateSensorModal(component.name.substr(7));
    this.UpdateSensorModal.show();
  }

  public closeUpdateSensorModal() {
    this.destroyUpdateSensorModal();
    this.UpdateSensorModal.hide();
  }

  public getSensorInfo({ component } : { component:Button }) {
    const idSensor = Number(component.name.substr(7));
    Http.get(`/ParamLeitContPessoas?NRPARAMLEITCONTPESSOAS=${idSensor}`).then((response: IDictionary) => {
      const sensor = response.data.dataset.Sensores[0];
      Metadata.getInstance<Select>('CDCLIENTE').setValue(sensor.CDCLIENTE);
      this.localId = sensor.NRLOCALCLIENTE; // Não pode salvar como null e não ta funcionando função do select
      Metadata.getInstance<Select>('IDTIPOSENSOR').setValue(Number(sensor.IDTIPOSENSOR));
      Metadata.getInstance<TextInput>('IDMACDEVICE').value = sensor.IDMACDEVICE;
      Metadata.getInstance<TextInput>('IDSENSOR').value = sensor.IDSENSOR;
      Metadata.getInstance<TextInput>('NRQTPESSOASALERT').value = sensor.NRQTPESSOASALERT;
      Metadata.getInstance<TextInput>('HRINICIODIA').value = sensor.HRINICIODIA;
      Metadata.getInstance<TextInput>('HRFINALDIA').value = sensor.HRFINALDIA;
      Metadata.getInstance<TextInput>('QTTEMPOREPETLEIT').value = sensor.QTTEMPOREPETLEIT;
      Metadata.getInstance<TextInput>('VRDISTANCIAMINLEIT').value = sensor.VRDISTANCIAMINLEIT;
      Metadata.getInstance<TextInput>('VRDISTANCIAMAXLEIT').value = sensor.VRDISTANCIAMAXLEIT;
      Metadata.getInstance<TextInput>('VRDISTANCIAMEDLEIT').value = sensor.VRDISTANCIAMEDLEIT;
      Metadata.getInstance<TextInput>('QTMATRIZLEITURA').value = sensor.QTMATRIZLEITURA;
      Metadata.getInstance<TextInput>('QTTEMPRECONHECIMENTO').value = sensor.QTTEMPRECONHECIMENTO;
      Metadata.getInstance<Select>('IDMODODISTANCIA').setValue(Number(sensor.IDMODODISTANCIA));
    });
  }

  public updateSensor({ component } : { component:Button }) {
    const form: Form = Metadata.getInstance('updateSensorForm');
    const validate = form.validate();
    if (validate) {
      const bodyPost: IDictionary = {};
      let fields = this.getGeneralFields();
      const sensorFields = this.getSensorExclusiveFields();
      if (!this.disabledSensorFields) {
        fields = fields.concat(sensorFields);
      } else {
        sensorFields.forEach((item) => {
          const nameValue: string = item.name;
          bodyPost[nameValue] = 0;
        });
      }
      fields.forEach((field) => {
        const nameValue: string = field.name;
        bodyPost[nameValue] = Number.isNaN(field.value) ? parseFloat(field.value) : field.value;
      });
      bodyPost.NRPARAMLEITCONTPESSOAS = component.name.substr(5); // Salva o id do sensor em um index de mesmo nome
      bodyPost.NRLOCALCLIENTE = 52;
      Http.post('/ParamLeitContPessoas/updateParamLeituras', bodyPost)
        .then((response: IDictionary) => {
          if (response.data.dataset.info.message === 'Editado com sucesso') {
            AlertService.show({
              name: 'updateSucceeded',
              text: 'Os parametros do sensor foram editados com sucesso.',
              top: true,
              left: false,
              timeout: 3000,
              color: 'success',
            });

            fields.forEach((field) => {
              field.value = '';
            });
            this.listSensor(); // reload nos sensores da tela
          } else {
            DialogService.show({
              name: 'updateFailed',
              type: 'error',
              title: 'Erro!',
              text: 'Não foi possível alterar as informações do setor',
            });
          }
        });
      this.closeUpdateSensorModal();
    }
  }
}
